//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//

#pragma once

#include "Control.h"

namespace VWFC
{
	namespace VWUI
	{
		// ---------------------------------------------------------------------------------------------------------------
		class VWTreeCtrlItem;

		// ---------------------------------------------------------------------------------------------------------------
		typedef int		TTreeItemID;

		// ---------------------------------------------------------------------------------------------------------------
		class VWTreeCtrlContainer
		{
			friend class VWTreeCtrlItem;
		protected:
									VWTreeCtrlContainer(TControlID dialogID, TControlID controlID, TTreeItemID itemID);
									VWTreeCtrlContainer(const VWTreeCtrlContainer& src);
			virtual					~VWTreeCtrlContainer();

			VWTreeCtrlContainer&	operator=(const VWTreeCtrlContainer& src);
			bool					operator==(const VWTreeCtrlContainer& src);
			bool					operator!=(const VWTreeCtrlContainer& src);

		public:
			bool			IsValid() const;

			VWTreeCtrlItem&	FindItem(TTreeItemID id);

			size_t			GetChildItemsCount() const;
			VWTreeCtrlItem&	GetChildItemAt(size_t index) const;
			bool			FindChildItemIndex(const VWTreeCtrlItem& itemToFind, VWTreeCtrlContainer*& outParentItem, size_t& outIndex);
			
			VWTreeCtrlItem&	AddChildItem(const TXString& text, bool addAtEnd = false);
			VWTreeCtrlItem&	InsertChildItem(const TXString& text, const VWTreeCtrlItem& afterItem);
			void			DeleteItem();

			//VWTreeCtrlItem&	GetParentItem();
			//VWTreeCtrlItem&	GetNextSiblingItem();
			//VWTreeCtrlItem&	GetPrevSiblingItem();

		public:
			static VWTreeCtrlItem	kEmptyTreeItem;

		protected:
			TControlID			fDialogID;
			TControlID			fThisControlID;
			TTreeItemID			fThisItemID;

			typedef std::vector<VWTreeCtrlItem*>		TItemsArray;
			TItemsArray			farrItems;
		};

		// ---------------------------------------------------------------------------------------------------------------
		class VWTreeCtrlItem : public VWTreeCtrlContainer
		{
			friend class VWTreeCtrlContainer;
		public:
							VWTreeCtrlItem();
							VWTreeCtrlItem(const VWTreeCtrlItem& src);
			virtual			~VWTreeCtrlItem();

			VWTreeCtrlItem&	operator=(const VWTreeCtrlItem& src);
			bool			operator==(const VWTreeCtrlItem& src);
			bool			operator!=(const VWTreeCtrlItem& src);

		public:
			TTreeItemID		GetItemID() const;

			void			ExpandItem(bool bExpand);
			void			SelectItem();
			void			SetItemData(long data);
			long			GetItemData() const;

		protected:
							VWTreeCtrlItem(TControlID dialogID, TControlID controlID, TTreeItemID itemID);

		};
	}
}