//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//
//

#include "StdHeaders.h"

#include "MiniCadExternals.h"
#include "CallbackIDs.h"
#include "CallbackParamTypes.h"
#include "MiniCadCallBacks.h"

// This must follow all header files.
#undef THIS_FILE
DECLARE_THIS_FILE



// =========================================================================================
// == XCall constructors and destructor

extern "C" {

//#if GS_USE_D0_RETURNS
//	#pragma pointers_in_D0
//#endif

	typedef long (*ExternalNameToIDPtr)(ConstStr31Param fileName);
	typedef GenericFunctionPtr (*OpenXCodePtr)(long extCodeInfoID, lboolean_t needsResourceFileOpen);
	typedef void (*CloseXCodePtr)(long extCodeInfoID);

//#if GS_USE_D0_RETURNS
//	#pragma pointers_in_A0
//#endif

}

//-------------------------------------------------------------------
XCall::XCall(CallBackPtr cbp, ConstStr31Param externalName, Boolean needsResourceFileOpen)
	: fCBB(cbp)
//
//	Constructor for XCall class.
//
{
	GenericFunctionPtr f1 = CallRetrieverProc(fCBB.rPtr, kcbExternalNameToID);
	GenericFunctionPtr f2 = CallRetrieverProc(fCBB.rPtr, kcbOpenXCode);

	if (f1) {
		fCodeID = (*(ExternalNameToIDPtr) f1)(externalName);
		fnPtr = f2 ? (*(OpenXCodePtr) f2)(fCodeID, needsResourceFileOpen) : nil;
	}
	else {
		fCodeID = -1;
		fnPtr = nil;
	}
}

//-------------------------------------------------------------------
XCall::XCall(CallBackPtr cbp, long externalID, Boolean needsResourceFileOpen)
	: fCBB(cbp)
//
//	Constructor for XCall class.
//
{
	GenericFunctionPtr f = CallRetrieverProc(fCBB.rPtr, kcbOpenXCode);
	
	fnPtr = f ? (*(OpenXCodePtr) f)(externalID, needsResourceFileOpen) : nil;
	fCodeID = externalID;
}

//-------------------------------------------------------------------
XCall::~XCall()
//
//	Destructor for XCall class.
//
{
	GenericFunctionPtr f = CallRetrieverProc(fCBB.rPtr, kcbCloseXCode);
	
	if (f && fCodeID != -1) (*(CloseXCodePtr) f)(fCodeID);
}

