//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//

#include "StdHeaders.h"

#include "ChooseClassPopupCtrl.h"

#include "Dialog.h"

using namespace VWFC::VWUI;

VWChooseClassPopupCtrl::VWChooseClassPopupCtrl(TControlID id)
	: VWControl( id )
{
}

VWChooseClassPopupCtrl::~VWChooseClassPopupCtrl()
{
}

bool VWChooseClassPopupCtrl::CreateControl(VWDialog* pDlg, short widthInCharacters)
{
	gSDK->CreateClassPullDownMenu( pDlg->GetDialogID(), fControlID, widthInCharacters);
	return VWControl::CreateControl( pDlg );
}

size_t VWChooseClassPopupCtrl::GetSelectedIndex() const
{
	short	nItemIndex	= -1;
	::GS_GetSelectedChoiceIndex( gCBP, fpParentDlg->GetControlID(), fControlID, nItemIndex );
	return (size_t) nItemIndex;
}

void VWChooseClassPopupCtrl::SelectIndex( size_t  nIndex )
{
	gSDK->SelectChoice( fpParentDlg->GetControlID(), fControlID, nIndex );	
}

void VWChooseClassPopupCtrl::GetSelectedClass( TXString & selectedClass ) const
{
	short nIndex;
	gSDK->GetSelectedChoiceIndex( fpParentDlg->GetControlID(), fControlID, nIndex) ;

	gSDK->GetChoiceText( fpParentDlg->GetControlID(), fControlID, nIndex, selectedClass );
}

void VWChooseClassPopupCtrl::SetSelectedClass( const TXString & selectedClass )
{
	GSStr255		szText;
	selectedClass.CopyInto( szText );

	bool bClassExists = false;
	MCObjectHandle hClass = gSDK->GetNamedObject( selectedClass );
	if ( hClass ) {
		if ( gSDK->GetObjectTypeN( hClass ) == kClassDefNode ) {
			bClassExists = true;
		}
	}

	
	short nItemIndex	= -1;
	if ( bClassExists ) {
		gSDK->GetChoiceIndex( fpParentDlg->GetControlID(), fControlID, szText, nItemIndex );
		if ( nItemIndex >= 0 ) {
			gSDK->SelectChoice( fpParentDlg->GetControlID(), fControlID, nItemIndex );
		}
	}
	else {
		gSDK->GetChoiceIndex( fpParentDlg->GetControlID(), fControlID, szText, nItemIndex );
		if ( nItemIndex == -1 ) {
			this->InsertProposedClass( selectedClass );
			gSDK->GetChoiceIndex( fpParentDlg->GetControlID(), fControlID, szText, nItemIndex );
		}
			
		if ( nItemIndex >= 0 ) {
			gSDK->SelectChoice( fpParentDlg->GetControlID(), fControlID, nItemIndex );
		}
	}

}

void VWChooseClassPopupCtrl::InsertProposedClass( const TXString & proposedClass )
{
#if T00809_IMAGE_RESOURCE_MANAGER_ENABLED
	gSDK->InsertProposedClassOrLayerItem( fpParentDlg->GetControlID(), fControlID, proposedClass, "");
#else
	gSDK->InsertProposedClassOrLayerItem( fpParentDlg->GetControlID(), fControlID, proposedClass, 0);
#endif

}


