#include "StdAfx.h"

#include "PluginResource.h"

#include "ExtObjClassName.h"
#include "ProviderShapePane.h"

using namespace TesterModule;

namespace TesterModule
{
	// --------------------------------------------------------------------------------------------------------
	static SToolDef		gToolDef = {
		/*ToolType*/					eExtensionToolType_DefaultPoint,
		/*ParametricName*/				"ProviderTestObj",
		/*PickAndUpdate*/				ToolDef::pickAndUpdate,
		/*NeedScreenPlane*/				ToolDef::doesntNeedScreenPlane,
		/*Need3DProjection*/			ToolDef::doesntNeed3DView,
		/*Use2DCursor*/					ToolDef::use2DCursor,
		/*ConstrainCursor*/				ToolDef::constrainCursor,
		/*NeedPerspective*/				ToolDef::doesntNeedPerspective,
		/*ShowScreenHints*/				ToolDef::showScreenHints,
		/*NeedsPlanarContext*/			ToolDef::needsPlanarContext,
		/*Reserved1*/					false,
		/*Reserved2*/					0,
		/*Reserved3*/					0,
		/*Reserved4*/					0,
		/*Message*/						{11000, 4},
		/*WaitMoveDistance*/			0,
		/*ConstraintFlags*/				0,
		/*BarDisplay*/					kToolBarDisplay_XYClLaZo,
		/*MinimumCompatibleVersion*/	900,
		/*Title*/						{11000, 2},
		/*Category*/					{11000, 3},
		/*HelpText*/					{11000},
		/*VersionCreated*/				16,
		/*VersoinModified*/				0,
		/*VersoinRetired*/				0,
		/*contextual help*/				"",
		/*Icon Specifier*/				"ProvidersSample/ExtObjClassNameDefTool.png",
		/*Cursor Specifier*/			""
	};

	// --------------------------------------------------------------------------------------------------------
	static SParametricDef gParametricDef = {
		/*LocalizedName*/			{11000, 1},
		/*SubType*/					kParametricSubType_Point,
		/*ResetOnMove*/				false,
		/*ResetOnRotate*/			false,
		/*WallInsertOnEdge*/		false,
		/*WallInsertNoBreak*/		false,
		/*WallInsertHalfBreak*/		false,
		/*WallInsertHideCaps*/		false,
	};

	static SParametricParamChc	gArrChoices[] = {
		{ 128,	"Line",					{11002, 1} },
		{ 128,	"Arc",					{11002, 2} },
		{ 128,	"Bezier",				{11002, 3} },
		{ 128,	"None",					{11002, 4} },
		//------
		{ 129,	"Default Legend",		{11002, 5} },
		//------
		// TERMINATE
		{ NULL, NULL, {0,0} }
	};

	static SParametricParamDef gArrParameters[] = {
		{ "p1",			{11001, 1},		"False",	"False",	kFieldBoolean,		0 },
		{ "p2",			{11001, 2},		"False",	"False",	kFieldBoolean,		0 },
		{ "p3",			{11001, 3},		"",			"",			kFieldText,			0 },
		{ "p4",			{11001, 4},		"",			"",			kFieldText,			0 },
		{ "p5",			{11001, 5},		"2\"",		"2\"",		kFieldCoordDisp,	0 },
		{ "p6",			{11001, 6},		"Auto",		"Auto",		kFieldPopUp,		128 },
		{ "p7",			{11001, 7},		"Auto",		"Auto",		kFieldPopUp,		129 },
		// TERMINATE
		{ NULL, {0,0}, NULL, NULL, EFieldStyle(0), 0 }
	};
}

// --------------------------------------------------------------------------------------------------------
// {56D3EBD2-4871-41BB-A9CF-EBBBC622F540}
IMPLEMENT_VWExtension(
	/*Extension class*/	CExtObjClassName,
	/*Event sink*/		CObjClassName_EventSink,
	/*Universal name*/	"ProviderTestObj",
	/*Version*/			1,
	/*UUID*/			0x56d3ebd2, 0x4871, 0x41bb, 0xa9, 0xcf, 0xeb, 0xbb, 0xc6, 0x22, 0xf5, 0x40 );

// Test with VectorScript:
// SetSelect(CreateCustomObject('ProviderTestObj',0,0,0));

// --------------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------------
CExtObjClassName::CExtObjClassName(CallBackPtr cbp)
	: VWExtensionParametric( cbp, gParametricDef, gArrParameters, gArrChoices )
{
}

CExtObjClassName::~CExtObjClassName()
{
}

void CExtObjClassName::DefineSinks()
{
	this->DefineSink<CProviderShapePane>( IID_ProviderShapePane );
//	this->DefineSink<CProviderReshape3D>( IID_ProviderReshape3D );
//	this->DefineSink<CProviderSiteModifiers>( IID_DTMModifiersProvider );
}

// --------------------------------------------------------------------------------------------------------
// {8252E086-52F3-44EF-AA81-9A5A386AC1D5}
IMPLEMENT_VWExtension(
	/*Extension class*/	CExtObjClassNameDefTool,
	/*Event sink*/		VWToolEmpty_EventSink,
	/*Universal name*/	"ProviderTestObjTool",
	/*Version*/			1,
	/*UUID*/			0x8252e086, 0x52f3, 0x44ef, 0xaa, 0x81, 0x9a, 0x5a, 0x38, 0x6a, 0xc1, 0xd5 );

// --------------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------------
CExtObjClassNameDefTool::CExtObjClassNameDefTool(CallBackPtr cbp)
	: VWExtensionTool( cbp, gToolDef )
{
}

CExtObjClassNameDefTool::~CExtObjClassNameDefTool()
{
}

// --------------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------------
CObjClassName_EventSink::CObjClassName_EventSink(IVWUnknown* parent)
	: VWParametric_EventSink( parent )
{
}

CObjClassName_EventSink::~CObjClassName_EventSink()
{
}

EObjectEvent CObjClassName_EventSink::OnInitXProperties(CodeRefID objectID)
{
	// obtain the interfact for accessing the extended properties
	using namespace VectorWorks::Extension;
	VCOMPtr<IExtendedProps>	extProps( IID_ExtendedProps );



	return kObjectEventNoErr;
}

EObjectEvent CObjClassName_EventSink::Recalculate()
{
	VWPoint3D	offset( -500, 0, 0 );

	VWPolygon3DObj	poly;
	poly.AddVertex( offset + VWPoint3D(50, 100, 0) );
	poly.AddVertex( offset + VWPoint3D(1000, 0, 0) );
	poly.AddVertex( offset + VWPoint3D(1000, 1000, 0) );
	poly.AddVertex( offset + VWPoint3D(100, 800, 0) );
	poly.SetClosed( true );

	return kObjectEventNoErr;
}
